<?php

/*
 *  This file is part of SplashSync Project.
 *
 *  Copyright (C) 2015-2019 Splash Sync  <www.splashsync.com>
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *  For the full copyright and license information, please view the LICENSE
 *  file that was distributed with this source code.
 */

namespace Splash\Local\Objects\Post;

use Splash\Client\Splash      as Splash;
use Splash\Local\Notifier;
use Splash\Local\Objects\Product\Variants\CoreTrait as Variants;
use WP_Post;

/**
 * Wordpress Post Hook Manager
 */
trait HooksTrait
{
    private static $postClass = "\\Splash\\Local\\Objects\\Post";

    /**
     * Register Post & Pages, Product Hooks
     */
    public static function registerHooks()
    {
        //====================================================================//
        // Setup Post Saved Hook
        $createCall = array( static::$postClass , "updated");
        if (is_callable($createCall)) {
            add_action('save_post', $createCall, 10, 3);
        }
        //====================================================================//
        // Setup Post Deleted Hook
        $deleteCall = array( static::$postClass , "deleted");
        if (is_callable($deleteCall)) {
            add_action('deleted_post', $deleteCall, 10, 1);
        }
    }

    /**
     * Main Post Updated Hook Action
     *
     * @param int     $postId
     * @param WP_Post $post
     * @param bool    $updated
     */
    public static function updated($postId, $post, $updated)
    {
        //====================================================================//
        // Stack Trace
        Splash::log()->trace();
        //====================================================================//
        // Check Id is Not Empty
        if (empty($postId)) {
            return;
        }
        //====================================================================//
        // Check Post is Not a Auto-Draft
        if ("auto-draft" == $post->post_status) {
            return;
        }
        //====================================================================//
        // Prepare Commit Parameters
        $action = $updated ? SPL_A_UPDATE : SPL_A_CREATE;
        $objectType = self::getSplashType($post);
        if (!is_string($objectType)) {
            return;
        }

        $comment = $objectType.($updated ? " Updated" : " Created")." on Wordpress";
        //====================================================================//
        // Catch Wc Actions on variable products
        if (("product" == $post->post_type) && did_action('woocommerce_init')) {
            $postId = Variants::getIdsForCommit($postId);
        }
        //====================================================================//
        // Check Commit is Allowed
        if (!self::isCommitAllowed($post->post_type, $objectType, $action)) {
            return;
        }
        //====================================================================//
        // Do Commit
        Splash::commit($objectType, $postId, $action, "Wordpress", $comment);
        //====================================================================//
        // Store User Messages
        Notifier::getInstance()->importLog();
    }

    /**
     * Detect Splash Object Type Name
     *
     * @param WP_Post $post
     *
     * @return boolean|string
     */
    public static function getSplashType($post)
    {
        switch ($post->post_type) {
            //====================================================================//
            // Core Wp Objects Types
            case "post":
                return "Post";
            case "page":
                return "Page";
            //====================================================================//
            // WooCommerce Objects Types
            case "product":
            case "product_variation":
                return "Product";
            case "shop_order":
                return "Order";
        }
        Splash::log()->deb("Unknown Object Type => ".$post->post_type);

        return false;
    }

    /**
     * Main Post Deleted Hook Action
     *
     * @param int $postId
     */
    public static function deleted($postId)
    {
        //====================================================================//
        // Stack Trace
        Splash::log()->trace();

        /** @var WP_Post $post */
        $post = get_post($postId);
        if ("post" == $post->post_type) {
            Splash::commit("Post", $postId, SPL_A_DELETE, "Wordpress", "Post Deleted");
        }
        if ("page" == $post->post_type) {
            Splash::commit("Page", $postId, SPL_A_DELETE, "Wordpress", "Page Deleted");
        }
        if ("product" == $post->post_type) {
            $postId = Variants::getIdsForCommit($postId);
            Splash::commit("Product", $postId, SPL_A_DELETE, "Wordpress", "Product Deleted");
        }
        if ("product_variation" == $post->post_type) {
            Splash::commit("Product", $postId, SPL_A_DELETE, "Wordpress", "Product Deleted");
        }
        if ("shop_order" == $post->post_type) {
            Splash::commit("Invoice", $postId, SPL_A_DELETE, "Wordpress", "Invoice Deleted");
            Splash::commit("Order", $postId, SPL_A_DELETE, "Wordpress", "Order Deleted");
        }
        //====================================================================//
        // Store User Messages
        Notifier::getInstance()->importLog();
    }

    /**
     * Detect Splash Object Type Name
     *
     * @param string $postType
     * @param string $objectType
     * @param string $action
     *
     * @return bool
     */
    private static function isCommitAllowed($postType, $objectType, $action)
    {
        //====================================================================//
        // Prevent Commit on Variant Product Create
        if (("product" == $postType)
                && (SPL_A_CREATE == $action)
                && Splash::object($objectType)->isLocked("onVariantCreate")) {
            return false;
        }
        //====================================================================//
        // Prevent Repeated Commit if Needed
        if ((SPL_A_UPDATE == $action) && Splash::object($objectType)->isLocked()) {
            return false;
        }

        return true;
    }
}
